@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.22 2003/11/08 23:43:25 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.29 2003/11/19 20:38:25 fuyu Exp $
  */
 
 #include "config.h"
@@ -133,7 +133,8 @@ typedef struct _RendererContext
   guint link_text_buffer_size;
   guint link_text_length;
 
-  gboolean last_char_is_greater_than;
+  int number_of_gt_characters;
+  gboolean last_text_is_link;
 
   GHashTable *tag_table;
 
@@ -182,21 +183,26 @@ PopupContext global_popup_context =
       end_element_cb,
       NULL,
       characters_cb
-    },
-    NULL,
-    NULL,
-    NULL,
-    -1,	/* ְ㤨ƥޡդʤ褦 */
+    },		/* handler */
+    NULL,	/* view */
+    NULL,	/* thread */
+    NULL,	/* application */
+    -1,		/* last_read_number: ְ㤨ƥޡդʤ褦 */
   
-    NULL,
-    popup_renderer_default_text_buffer,
-    DEFAULT_BUFFER_SIZE,
-    0,
+    NULL,				/* link */
+    popup_renderer_default_text_buffer,	/* link_text_buffer */
+    DEFAULT_BUFFER_SIZE,		/* link_text_buffer_size */
+    0,					/* link_text_length */
 
-    NULL,
-    NULL,
-    NULL,
-    NULL,
+    0,					/* number_of_gt_characters */
+    FALSE,				/* last_text_is_link */
+
+    NULL,				/* tag_table */
+
+    NULL,				/* thread_title_tag */
+    NULL,				/* response_name_tag */
+    NULL,				/* response_paragraph_tag */
+    NULL,				/* bold_tag */
   },
 
   NULL,		/* popup_window */
@@ -230,8 +236,8 @@ static GQuark popup_context_id;
 static GQuark popup_parent_context_id;
 
 
-#if ENABLE_REGEXP
-static regex_t head_number_pattern;
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+static regex_t comma_separated_link_pattern;
 static regex_t informal_link_pattern;
 #endif
 
@@ -398,16 +404,18 @@ initialize_thread_ui(OchushaApplication *application)
   initialize_popup_context(&global_popup_context, application);
 #endif
 
-#if ENABLE_REGEXP
-  if (regcomp(&head_number_pattern,
-	      "^([1-9][0-9]*)", REG_EXTENDED))
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
+  if (regcomp(&comma_separated_link_pattern,
+	      "^[,\357\274\214\343\200\201][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
+	      REG_EXTENDED))
     {
       fprintf(stderr, "invalid regular expression\n");
     }
 
-  /* "\076\357\274\236\342\211\253"  ">" UTF-8ɽ */
+  /* "\357\274\236\342\211\253"  "" UTF-8ɽ */
   if (regcomp(&informal_link_pattern,
-	      "[\076\357\274\236\342\211\253]+([1-9][0-9]*)",
+	      "[>\357\274\236\342\211\253]+[ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
 	      REG_EXTENDED))
     {
       fprintf(stderr, "invalid regular expression\n");
@@ -812,20 +820,21 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 	end_element_cb,
 	NULL,
 	characters_cb
-      },
-      job_args->view,
-      thread,
-      application,
-      thread->number_of_responses_read,
+      },					/* handler */
+      job_args->view,				/* view */
+      thread,					/* thread */
+      application,				/* application */
+      thread->number_of_responses_read,		/* last_read_number */
 
-      NULL,
-      default_text_buffer,
-      DEFAULT_BUFFER_SIZE,
-      0,
+      NULL,					/* link */
+      default_text_buffer,			/* link_text_buffer */
+      DEFAULT_BUFFER_SIZE,			/* link_text_buffer_size */
+      0,					/* link_text_length */
 
-      FALSE,
+      0,					/* number_of_gt_characters */
+      FALSE,					/* last_text_is_link */
 
-      tag_table,
+      tag_table,				/* tag_table */
 
       bbs_thread_view_get_tag_by_name(job_args->view, "thread_title"),
       bbs_thread_view_get_tag_by_name(job_args->view, "response_name"),
@@ -1216,6 +1225,7 @@ process_link(RendererContext *context)
 				      context->link);
   G_FREE(context->link);
   context->link = NULL;
+  context->last_text_is_link = TRUE;
 }
 
 
@@ -1226,6 +1236,7 @@ start_element_cb(void *user_data, const char *name, const char *const attrs[])
   BBSThreadView *view = context->view;
   int i;
 
+  context->last_text_is_link = FALSE;
 #if DEBUG_GUI_MOST
   fprintf(stderr, "<%s", name);
   i = 0;
@@ -1236,6 +1247,13 @@ start_element_cb(void *user_data, const char *name, const char *const attrs[])
     }
   fprintf(stderr, ">\n");
 #endif
+
+  while (context->number_of_gt_characters > 0)
+    {
+      bbs_thread_view_append_text(context->view, ">", 1);
+      context->number_of_gt_characters--;
+    }
+
   bbs_thread_view_push_tag(view,
 			   g_hash_table_lookup(context->tag_table, name));
 
@@ -1288,6 +1306,12 @@ end_element_cb(void *user_data, const char *name)
 #if DEBUG_GUI_MOST
   fprintf(stderr, "</%s>\n", name);
 #endif
+  while (context->number_of_gt_characters > 0)
+    {
+      bbs_thread_view_append_text(context->view, ">", 1);
+      context->number_of_gt_characters--;
+    }
+
   bbs_thread_view_pop_tags(view,
 			   g_hash_table_lookup(context->tag_table, name));
 
@@ -1305,82 +1329,213 @@ characters_cb(void *user_data, const char *ch, int len)
 
   if (context->link == NULL)
     {
-#if ENABLE_REGEXP
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
       if (context->application->analyze_informal_links)
 	{
-	  regmatch_t match[2];
-	  gchar *text = g_strndup(ch, len);
-	  gchar *text_top = text;
+	  regmatch_t match[3];
+	  gchar *text;
+	  gchar *text_top;
 
-	  if (context->last_char_is_greater_than)
+	  if (*ch == '>' && len == 1)
 	    {
-	      if (regexec(&head_number_pattern, text, 1,
-			  match, 0) == 0)
-		{
-		  int number;
-		  int match_len = match[1].rm_eo - match[1].rm_so;
-		  if (sscanf(text, "%d", &number) == 1)
-		    {
-		      char *link = ochusha_bbs_thread_get_url_for_response(context->thread, number);
-		      if (link != NULL)
-			{
-			  bbs_thread_view_append_text_as_link(context->view,
-							      text, match_len,
-							      link);
-			  g_free(link);
-			  text_top += match_len;
-			  len -= match_len;
-			}
-		    }
-		}
+	      context->number_of_gt_characters++;
+	      len = 0;
+	      text = NULL;
 	    }
-
-	  while (regexec(&informal_link_pattern, text_top, 1, match, 0) == 0)
+	  else if (context->number_of_gt_characters > 0)
 	    {
-	      int no_match_len = match[0].rm_so;
-	      int match_len = match[0].rm_eo - match[0].rm_so;
-	      int number;
-	      char *link;
-
-	      if (no_match_len > 0)
-		bbs_thread_view_append_text(context->view, text_top,
-					    no_match_len);
-	      if (sscanf(text_top + match[1].rm_so, "%d", &number) == 1)
-		link = ochusha_bbs_thread_get_url_for_response(context->thread, number);
-	      else
-		link = NULL;
-
-	      if (link != NULL)
-		{
-		  bbs_thread_view_append_text_as_link(context->view,
-						      text_top + match[0].rm_so,
-						      match_len,
-						      link);
-		  g_free(link);
-		}
-	      else
-		bbs_thread_view_append_text(context->view,
-					    text_top + match[0].rm_so,
-					    match_len);
-
-	      text_top += no_match_len + match_len;
-	      len -= (no_match_len + match_len);
+#if 0
+	      if (len > 0)
+		fprintf(stderr, "XXX: text=\"%s\", len=%d\n", ch, len);
+#endif
+	      text = G_MALLOC(len + context->number_of_gt_characters + 1);
+	      memset(text, '>', context->number_of_gt_characters);
+	      memcpy(text + context->number_of_gt_characters, ch, len);
+#if 0
+	      strncpy(text + context->number_of_gt_characters, ch, len);
+#endif
+	      text_top = text;
+	      len += context->number_of_gt_characters;
+	      text[len] = '\0';
+	      context->number_of_gt_characters = 0;
+#if 0
+	      if (len > 0)
+		fprintf(stderr, "YYY: text=\"%s\", len=%d\n", text, len);
+#endif
 	    }
-
-	  if (len > 0)
-	    bbs_thread_view_append_text(context->view, text_top, len);
-	  g_free(text);
-
-	  if (ch[len - 1] == '>')
-	    context->last_char_is_greater_than = TRUE;
 	  else
-	    context->last_char_is_greater_than = FALSE;
+	    text = g_strndup(ch, len);
+
+#if 0
+	  if (len > 0)
+	    fprintf(stderr, "ZZZ: text=\"%s\", len=%d\n", text, len);
+#endif
+	  text_top = text;
+
+	  while (len > 0)
+	    {
+	      while (context->last_text_is_link
+		     && regexec(&comma_separated_link_pattern,
+				text_top, 2, match, 0) == 0)
+		{
+		  int match_len = match[0].rm_eo - match[0].rm_so;
+		  int from = 0;
+		  int to = 0;
+		  char *link;
+
+		  if (match[1].rm_so != -1 && match[1].rm_eo != match[1].rm_so)
+		    {
+		      char *from_string = g_strndup(text_top + match[1].rm_so,
+						    match[1].rm_eo - match[1].rm_so);
+		      sscanf(from_string, "%d", &from);
+		      g_free(from_string);
+		    }
+
+		  if (match[2].rm_so != -1 && match[2].rm_eo != match[2].rm_so)
+		    {
+		      char *to_string = g_strndup(text_top + match[2].rm_so + 1,
+						  match[2].rm_eo - match[2].rm_so - 1);
+		      sscanf(to_string, "%d", &to);
+		      g_free(to_string);
+		    }
+		  else
+		    to = from;
+#if 0
+		  if (match[1].rm_so == -1 || match[1].rm_eo == match[1].rm_so
+		      || sscanf(text_top + match[1].rm_so, "%d", &from) != 1)
+		    from = 0;
+
+		  if (match[2].rm_so == -1 || match[2].rm_eo == match[2].rm_so
+		      || sscanf(text_top + match[2].rm_so + 1, "%d", &to) != 1)
+		    to = from;
+#endif
+
+		  if (from > 0)
+		    link = ochusha_bbs_thread_get_url_for_response(context->thread,
+								   from, to);
+		  else
+		    link = NULL;
+
+		  if (link != NULL)
+		    {
+		      bbs_thread_view_append_text_as_link(context->view,
+							  text_top + match[0].rm_so,
+							  match_len,
+							  link);
+		      g_free(link);
+		    }
+		  else
+		    bbs_thread_view_append_text(context->view,
+						text_top + match[0].rm_so,
+						match_len);
+
+		  text_top += match_len;
+		  len -= match_len;
+		}
+
+	      context->last_text_is_link = FALSE;
+
+	      if (len == 0)
+		break;
+
+	      if (regexec(&informal_link_pattern, text_top, 2, match, 0) == 0)
+		{
+		  int no_match_len = match[0].rm_so;
+		  int match_len = match[0].rm_eo - match[0].rm_so;
+		  int from;
+		  int to;
+		  char *link;
+
+#if 0
+		  fprintf(stderr, "ZZZ: len=%d, no_match_len=%d, match_len=%d\n",
+			  len, no_match_len, match_len);
+#endif
+
+		  if (no_match_len > 0)
+		    bbs_thread_view_append_text(context->view, text_top,
+						no_match_len);
+
+		  if (match[1].rm_so != -1 && match[1].rm_eo != match[1].rm_so)
+		    {
+		      char *from_string = g_strndup(text_top + match[1].rm_so,
+						    match[1].rm_eo - match[1].rm_so);
+		      sscanf(from_string, "%d", &from);
+		      g_free(from_string);
+		    }
+
+		  if (match[2].rm_so != -1 && match[2].rm_eo != match[2].rm_so)
+		    {
+		      char *to_string = g_strndup(text_top + match[2].rm_so + 1,
+						  match[2].rm_eo - match[2].rm_so - 1);
+		      sscanf(to_string, "%d", &to);
+		      g_free(to_string);
+		    }
+		  else
+		    to = from;
+#if 0
+		  if (match[1].rm_so == -1 || match[1].rm_eo == match[1].rm_so
+		      || sscanf(text_top + match[1].rm_so, "%d", &from) != 1)
+		    from = 0;
+
+		  if (match[2].rm_so == -1 || match[2].rm_eo == match[2].rm_so
+		      || sscanf(text_top + match[2].rm_so + 1, "%d", &to) != 1)
+		    to = from;
+#endif
+
+		  if (from > 0)
+		    link = ochusha_bbs_thread_get_url_for_response(context->thread,
+								   from, to);
+		  else
+		    link = NULL;
+
+		  if (link != NULL)
+		    {
+		      bbs_thread_view_append_text_as_link(context->view,
+							  text_top + match[0].rm_so,
+							  match_len,
+							  link);
+		      g_free(link);
+		      context->last_text_is_link = TRUE;
+		    }
+		  else
+		    bbs_thread_view_append_text(context->view,
+						text_top + match[0].rm_so,
+						match_len);
+
+		  text_top += no_match_len + match_len;
+		  len -= (no_match_len + match_len);
+		}
+	      else
+		{
+		  if (text_top[len - 1] == '>')
+		    {
+		      context->number_of_gt_characters++;
+		      len--;
+		    }
+
+		  if (len > 0)
+		    {
+#if 0
+		      fprintf(stderr, "CHECKPOINT #1\n");
+		      fprintf(stderr, "Appending text: \"%s\", len=%d\n",
+			      text_top, len);
+#endif
+		      bbs_thread_view_append_text(context->view, text_top, len);
+#if 0
+		      fprintf(stderr, "CHECKPOINT #2\n");
+#endif
+		    }
+
+		  break;
+		}
+	    }
+
+	  if (text != NULL)
+	    g_free(text);
 	}
-      else
-	bbs_thread_view_append_text(context->view, ch, len);
 #else
       bbs_thread_view_append_text(context->view, ch, len);
-#endif
+#endif	/* defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA) */
       return;
     }
 
