@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.16 2003/11/08 23:31:16 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.17 2003/11/19 11:33:00 fuyu Exp $
  */
 
 #include "config.h"
@@ -920,9 +920,31 @@ bbs_thread_view_append_text(BBSThreadView *view, const gchar *text, int len)
   args.buffer = view->text_buffer;
   start_offset = gtk_text_iter_get_offset(&view->iter);
 
-  gtk_text_buffer_insert(view->text_buffer, &view->iter, text, len);
+  if (view->last_is_new_line)
+    {
+      if (len > 0)
+	while (len > 0 && *text == ' ')
+	  {
+	    text++;
+	    len--;
+	  }
+      else
+	while (*text != '\0' && *text == ' ')
+	  {
+	    text++;
+	  }
+    }
+
+  if (len != 0)
+    gtk_text_buffer_insert(view->text_buffer, &view->iter, text, len);
+
   gtk_text_buffer_get_iter_at_offset(view->text_buffer, &start, start_offset);
   g_slist_foreach(view->tags, apply_tag, &args);
+
+  if (text[len - 1] == '\n')
+    view->last_is_new_line = 1;
+  else
+    view->last_is_new_line = 0;
 }
 
 
