@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.42 2003/11/08 20:02:47 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.45 2003/11/19 21:15:57 fuyu Exp $
  */
 
 #include "config.h"
@@ -663,7 +663,9 @@ typedef struct _OpenThreadJobArgs
 {
   OchushaApplication *application;
   OchushaBulletinBoard *board;
+#if 0	/* XXX: not used */
   OchushaBBSThread *thread;
+#endif
   char *thread_id;
   int res_num;
   gboolean in_tab;
@@ -672,8 +674,8 @@ typedef struct _OpenThreadJobArgs
 
 void
 ochusha_open_thread(OchushaApplication *application,
-		    OchushaBulletinBoard *board, OchushaBBSThread *thread,
-		    const char *thread_id, int res_num, gboolean in_tab)
+		    OchushaBulletinBoard *board, const char *thread_id,
+		    int res_num, gboolean in_tab)
 {
   WorkerJob *job;
   OpenThreadJobArgs *args;
@@ -697,7 +699,9 @@ ochusha_open_thread(OchushaApplication *application,
 
   args->application = application;
   args->board = board;
+#if 0	/* XXX: not used */
   args->thread = thread;
+#endif
   args->thread_id = G_STRDUP(thread_id);
   args->res_num = res_num;
   args->in_tab = in_tab;
@@ -1671,8 +1675,9 @@ write_dialog_response_cb(GtkWidget *write_dialog, gint response_id,
 	      BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(thread);
 	      BulletinBoardGUIInfo *board_info
 		= ensure_bulletin_board_info(thread->board, application);
-	      gboolean number_only = reverse_strpbrk(response.name,
-						     "0123456789") == NULL;
+	      gboolean number_only
+		= (response.name[0] != '\0'
+		   && reverse_strpbrk(response.name, "0123456789") == NULL);
 
 	      if (thread_info->last_name == NULL
 		  || strcmp(thread_info->last_name, response.name) != 0)
@@ -1776,6 +1781,14 @@ write_response(OchushaApplication *application)
   if (view == NULL || thread == NULL)
     return;
 
+  if (!ochusha_bbs_thread_is_post_supported(thread))
+    {
+      ochusha_open_url(application,
+		       ochusha_bbs_thread_get_url_to_post_response(thread),
+		       FALSE, TRUE);
+      return;
+    }
+
   info = ensure_bbs_thread_info(thread);
 
   if (info->write_dialog == NULL)
