@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.h,v 1.10 2003/11/04 07:59:09 fuyu Exp $
+ * $Id: bulletin_board_ui.h,v 1.11 2003/11/19 20:07:07 fuyu Exp $
  */
 
 #ifndef _BULLETIN_BOARD_UI_H_
@@ -94,9 +94,8 @@ void finalize_bulletin_board_view(OchushaBulletinBoard *board,
 				  GtkWidget *widget);
 
 void ochusha_open_thread(OchushaApplication *application,
-			 OchushaBulletinBoard *board, OchushaBBSThread *thread,
-			 const char *thread_id, int res_num,
-			 gboolean in_tab);
+			 OchushaBulletinBoard *board, const char *thread_id,
+			 int res_num, gboolean in_tab);
 
 void refresh_threadlist_view(OchushaApplication *application,
 			     OchushaBulletinBoard *board);
