@@ -64,7 +64,15 @@ extern void ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:12) */
+/* VOID:ENUM,INT,BOOLEAN (marshal.list:12) */
+extern void ochusha_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
+                                                    GValue       *return_value,
+                                                    guint         n_param_values,
+                                                    const GValue *param_values,
+                                                    gpointer      invocation_hint,
+                                                    gpointer      marshal_data);
+
+/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:13) */
 extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                                    GValue       *return_value,
                                                                    guint         n_param_values,
@@ -72,7 +80,7 @@ extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure
                                                                    gpointer      invocation_hint,
                                                                    gpointer      marshal_data);
 
-/* BOOLEAN:ENUM,INT (marshal.list:13) */
+/* BOOLEAN:ENUM,INT (marshal.list:14) */
 extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -80,7 +88,7 @@ extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT (marshal.list:14) */
+/* BOOLEAN:OBJECT (marshal.list:15) */
 extern void ochusha_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -88,7 +96,7 @@ extern void ochusha_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:15) */
+/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:16) */
 extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -96,7 +104,7 @@ extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
-/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:16) */
+/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:17) */
 extern void ochusha_marshal_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                                          GValue       *return_value,
                                                                          guint         n_param_values,
