@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.18 2003/11/08 23:43:25 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.19 2003/11/20 03:14:37 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -45,6 +45,8 @@
 # if HAVE_ONIGURUMA
 #  if HAVE_ONIG_ONIGPOSIX_H
 #    include <onig/onigposix.h>
+#  elif HAVE_ONIGPOSIX_H
+#    include <onigposix.h>
 #  else
 #    error "Use of oniguruma without onig/onigposix.h header isn't considered."
 #  endif
