@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.5 2003/11/02 21:30:00 fuyu Exp $
+ * $Id: htmlutils.c,v 1.6 2003/11/18 21:02:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -506,7 +506,7 @@ parse_text(ElementHandler *handler, void *context, const char *text, int len)
 
       if (cur_pos != tmp_pos && handler->characters != NULL)
 	{
-	  if (handler->converter != 0)
+	  if (handler->converter != NULL)
 	    {
 	      const char *inbuf = cur_pos;
 	      gchar *outbuf = buffer;
