@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.11 2003/11/08 23:44:33 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.13 2003/11/18 21:02:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -484,7 +484,10 @@ ochusha_bbs_thread_is_post_supported(OchushaBBSThread *thread)
   g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread), FALSE);
 
   klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
-  return klass->post_response != NULL;
+  if (klass->post_supported != NULL)
+    return (*klass->post_supported)(thread);
+
+  return FALSE;
 }
 
 
@@ -520,7 +523,8 @@ ochusha_bbs_thread_get_url(OchushaBBSThread *thread)
 
 
 char *
-ochusha_bbs_thread_get_url_for_response(OchushaBBSThread *thread, int number)
+ochusha_bbs_thread_get_url_for_response(OchushaBBSThread *thread, int from,
+					int to)
 {
   OchushaBBSThreadClass *klass;
   g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread), NULL);
@@ -528,7 +532,7 @@ ochusha_bbs_thread_get_url_for_response(OchushaBBSThread *thread, int number)
   klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
 
   if (klass->get_url_for_response != NULL)
-    return (*klass->get_url_for_response)(thread, number);
+    return (*klass->get_url_for_response)(thread, from, to);
   else
     return NULL;
 }
@@ -621,12 +625,6 @@ ochusha_bbs_thread_write_threadlist_element(OchushaBBSThread *thread,
 {
   OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread, id);
   OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread, title);
-#if 1	/* for backward compatibility */
-  fprintf(threadlist_xml,			
-	  "      <attribute name=\"dat_filename\">\n"			
-	  "        <string>%s</string>\n"
-	  "      </attribute>\n", thread->id);
-#endif
   OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, thread,
 			      number_of_responses_read);
   OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, thread, flags);
