@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.12 2003/11/04 01:42:04 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.13 2003/11/18 21:02:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -193,6 +193,7 @@ ochusha_bulletin_board_class_init(OchushaBulletinBoardClass *klass)
   klass->threadlist_read_thread_element = NULL;
   klass->threadlist_write_thread_element = NULL;
 
+  klass->generate_base_path = NULL;
   klass->generate_board_id = ochusha_bulletin_board_generate_board_id;
   klass->thread_new = NULL;
   klass->lookup_thread_by_url = NULL;
@@ -372,9 +373,6 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
   server = ochusha_utils_url_extract_http_server(url);
   g_return_if_fail(server != NULL);
 
-  base_path = ochusha_utils_url_extract_http_absolute_path(url);
-  g_return_if_fail(base_path != NULL);
-
   if (board->base_url != NULL)
     G_FREE(board->base_url);
   board->base_url = G_STRDUP(url);
@@ -383,6 +381,14 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
     G_FREE(board->server);
   board->server = server;
 
+  klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
+
+  if (*klass->generate_base_path)
+    base_path = (*klass->generate_base_path)(board, url);
+  else
+    base_path = ochusha_utils_url_extract_http_absolute_path(url);
+  g_return_if_fail(base_path != NULL);
+
   if (board->base_path != NULL)
     G_FREE(board->base_path);
   board->base_path = base_path;
@@ -390,7 +396,45 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
   if (board->id != NULL)
     G_FREE(board->id);
 
+  board->id = (*klass->generate_board_id)(board, url);
+}
+
+
+void
+ochusha_bulletin_board_set_bbs_type(OchushaBulletinBoard *board,
+				    OchushaBBSType bbs_type)
+{
+  OchushaBulletinBoardClass *klass;
+  char *url = board->base_url;
+  char *server;
+  char *base_path;
+
+  g_return_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board));
+
+  board->bbs_type = bbs_type;
+
+  server = ochusha_utils_url_extract_http_server(url);
+  g_return_if_fail(server != NULL);
+
+  if (board->server != NULL)
+    G_FREE(board->server);
+  board->server = server;
+
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
+
+  if (*klass->generate_base_path)
+    base_path = (*klass->generate_base_path)(board, url);
+  else
+    base_path = ochusha_utils_url_extract_http_absolute_path(url);
+  g_return_if_fail(base_path != NULL);
+
+  if (board->base_path != NULL)
+    G_FREE(board->base_path);
+  board->base_path = base_path;
+
+  if (board->id != NULL)
+    G_FREE(board->id);
+
   board->id = (*klass->generate_board_id)(board, url);
 }
 
@@ -852,17 +896,29 @@ endElementHandler(void *context, const xmlChar *name)
 	  OchushaBulletinBoard *board = sax_context->board;
 	  GHashTable *thread_attrs = sax_context->thread_attributes;
 
-	  char *thread_id = g_hash_table_lookup(thread_attrs, "thread_id");
+	  char *thread_id = g_hash_table_lookup(thread_attrs, "id");
+
+	  /* for backward compatibility */
 	  if (thread_id == NULL)
-	    {
-	      thread_id = g_hash_table_lookup(thread_attrs, "dat_filename");
-	    }
+	    thread_id = g_hash_table_lookup(thread_attrs, "dat_filename");
 
 	  if (thread_id != NULL)
 	    {
-	      OchushaBBSThread *thread
-		= ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
-								 thread_id);
+	      OchushaBBSThread *thread;
+
+	      /* for backward compatibility */
+	      char *tmp_id;
+	      char *tmp_pos;
+	      if ((tmp_pos = strstr(thread_id, ".dat")) != NULL)
+		{
+		  tmp_id = G_STRNDUP(thread_id, tmp_pos - thread_id);
+		  thread_id = tmp_id;
+		}
+	      else
+		tmp_id = NULL;
+
+	      thread = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
+								thread_id);
 
 	      if (thread == NULL)
 		{
@@ -884,6 +940,8 @@ endElementHandler(void *context, const xmlChar *name)
 				    thread_attrs);
 		    }
 		}
+	      if (tmp_id != NULL)
+		G_FREE(tmp_id);
 	    }
 	   /* ꡼󥢥å */
 	  hash_table_cleanup(thread_attrs);
