@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.8 2003/11/04 07:55:05 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.10 2003/11/19 21:37:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -428,7 +428,7 @@ get_content_encoding_type(ghttp_request *request)
     {
       if (strcmp(encoding, "gzip") == 0)
 	return GZIP_ENCODING;
-#if 1 || DEBUG_NETWORK
+#if DEBUG_NETWORK
       fprintf(stderr, "Content-Encoding: %s\n", encoding);
 #endif
       if (strcmp(encoding, "deflate") == 0)
@@ -682,8 +682,14 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	ghttp_flush_response_buffer(request);
       else if (state == ghttp_error)
 	{
-	  error_message = _("Unknown I/O error.");
-	  goto error_exit;
+	  /* XXX:  */
+	  status_code = ghttp_status_code(request);
+	  if (status_code != 200)
+	    {
+	      error_message = _("Unknown I/O error.");
+	      goto error_exit;
+	    }
+	  state = ghttp_done;
 	}
 
       body = ghttp_get_body(request);
@@ -794,7 +800,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
   if (status_code == 200)
     {
-      
       const char *tmp_header
 	= ghttp_get_header(request, http_hdr_Last_Modified);
       if (tmp_header != NULL)
@@ -831,6 +836,8 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 		    error_message,
 		    &signal_result);
+      fprintf(stderr, "access failed: status_code=%d %s\n",
+	      status_code, error_message);
     }
   else if (status_code != 0)
     {
@@ -844,6 +851,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 		    message,
 		    &signal_result);
+      fprintf(stderr, "%s\n", message);
     }
   else
     g_signal_emit(G_OBJECT(broker),
@@ -1869,6 +1877,304 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 }
 
 
+static gboolean
+http_read_from_url_synchronously(OchushaNetworkBroker *broker,
+				 OchushaAsyncBuffer *buffer)
+{
+  gboolean signal_result;
+  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
+					      broker_job_args_id);
+  OchushaNetworkBrokerBufferStatus *status
+    = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
+  const char *url = args->url;
+  const char *error_message = NULL;
+  const char *body;
+  int body_len;
+  int status_code = 0;
+  ghttp_request *request = NULL;
+  ContentEncodingType content_encoding = UNKNOWN_ENCODING;
+  GzipBuffer *gzip_buffer = NULL;
+  ghttp_status state;
+
+  request = ghttp_request_new();
+  args->request = request;	/* 塹Τˡġ */
+
+  ghttp_set_uri(request, (char *)url);
+  ghttp_set_type(request, ghttp_type_get);
+  ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
+  ghttp_set_header(request, http_hdr_Connection, "close");
+
+  if (!setup_common_request_headers(broker->config, request))
+    {
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
+		    _("Proxy setting may be wrong."),
+		    &signal_result);
+
+      ghttp_request_destroy(request);
+      args->request = NULL;
+      return FALSE;
+    }    
+
+  g_signal_emit(G_OBJECT(broker),
+		broker_signals[ACCESS_STARTED_SIGNAL],
+		0,
+		buffer,
+		&signal_result);
+
+  ghttp_set_sync(request, ghttp_sync);
+  ghttp_prepare(request);
+  state = ghttp_process(request);
+
+  if (state == ghttp_error)
+    {
+      error_message = _("Unknown I/O error.");
+      goto error_exit;
+    }
+
+  status_code = ghttp_status_code(request);
+  if (status_code != 200)
+    {
+      goto error_exit;
+    }
+
+  body = ghttp_get_body(request);
+  body_len = ghttp_get_body_len(request);
+      
+  if (body_len > 0)
+    {
+      ghttp_current_status current_status = ghttp_get_status(request);
+      if (current_status.bytes_total > 0)
+	g_signal_emit(G_OBJECT(broker),
+		      broker_signals[ACCESS_PROGRESSED_SIGNAL],
+		      0,
+		      buffer,
+		      current_status.bytes_read,
+		      current_status.bytes_total,
+		      &signal_result);
+
+      content_encoding = get_content_encoding_type(request);
+
+      switch (content_encoding)
+	{
+	case GZIP_ENCODING:
+	  gzip_buffer = gzip_buffer_new(buffer);
+	  if (gzip_buffer == NULL)
+	    {
+	      error_message = _("Out of memory.");
+	      goto error_exit;
+	    }
+	  break;
+
+	case DEFLATE_ENCODING:
+	  gzip_buffer = gzip_buffer_new(buffer);
+	  if (gzip_buffer == NULL)
+	    {
+	      error_message = _("Out of memory.");
+	      goto error_exit;
+	    }
+	  gzip_buffer_skip_gzip_header(gzip_buffer);
+	  break;
+
+	case UNKNOWN_ENCODING:
+	  error_message = _("Unknown transfer encoding.");
+	  goto error_exit;
+  
+	case IDENTITY_ENCODING:
+	  break;
+	}
+
+      if (content_encoding == GZIP_ENCODING
+	  || content_encoding == DEFLATE_ENCODING)
+	{
+	  GzipBufferStatus result
+	    = gzip_buffer_append_data(gzip_buffer, body, body_len);
+	  if (result == GZIP_BUFFER_ERROR)
+	    {
+	      error_message = _("Inflation failed.");
+	      gzip_buffer_free(gzip_buffer);
+	      goto error_exit;
+	    }
+	  if (result == GZIP_BUFFER_INFLATION_DONE)
+	    {
+	      gzip_buffer_free(gzip_buffer);
+	      gzip_buffer = NULL;
+#if DEBUG_NETWORK
+	      if (state != ghttp_done)
+		fprintf(stderr, "Requested file contains garbage?\n");
+#endif
+	    }
+	}
+      else if (!ochusha_async_buffer_append_data(buffer, body, body_len,
+		"ochusha_network_broker.c: http_read_from_url_synchronously"))
+	{	/* content_encoding == IDENTITY_ENCODING */
+	  g_signal_emit(G_OBJECT(broker),
+			broker_signals[ACCESS_TERMINATED_SIGNAL],
+			0,
+			buffer,
+			&signal_result);
+	  ghttp_request_destroy(request);
+	  args->request = NULL;
+
+	  status->state
+	    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+
+	  return FALSE;
+	}
+    }
+
+  if (gzip_buffer != NULL)
+    {
+#if DEBUG_NETWORK
+      fprintf(stderr, "What's happen?  Connection was closed but inflation hasn't been completed.\n");
+#endif
+      gzip_buffer_free(gzip_buffer);
+    }
+
+#if DEBUG_NETWORK_MOST
+  fprintf(stderr, "http_read_from_url_synchronously succeeded.\n");
+#endif
+
+  return TRUE;
+
+ error_exit:
+#if DEBUG_NETWORK
+  fprintf(stderr, "force_read_synchronously(): error.\n");
+#endif
+  if (error_message != NULL)
+    {
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+		    error_message,
+		    &signal_result);
+      fprintf(stderr, "access failed: status_code=%d %s\n",
+	      status_code, error_message);
+    }
+  else if (status_code != 0)
+    {
+      char message[4096];
+      snprintf(message, 4096, "Access failed: %d (%s)",
+	       status_code, ghttp_reason_phrase(request));
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+		    message,
+		    &signal_result);
+      fprintf(stderr, "%s\n", message);
+    }
+  else
+    g_signal_emit(G_OBJECT(broker),
+		  broker_signals[ACCESS_FAILED_SIGNAL],
+		  0,
+		  buffer,
+		  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+		  _("Access failed: unknown reason."),
+		  &signal_result);
+  if (request != NULL)
+    ghttp_request_destroy(request);
+  args->request = NULL;
+
+  return FALSE;
+
+}
+
+
+static void
+force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
+{
+  gboolean signal_result;
+  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
+					      broker_job_args_id);
+  OchushaNetworkBrokerBufferStatus *status
+    = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
+
+#if DEBUG_THREAD_MOST
+  fprintf(stderr, "force_read_synchronously() thread is invoked.\n");
+#endif
+
+  if (!ochusha_async_buffer_active_ref(buffer,
+		"ochusha_network_broker.c: force_read_synchronously()"))
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+      g_signal_emit(G_OBJECT(args->broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+      goto finish_force_read;
+    }
+
+  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
+
+  if (args->if_modified_since != NULL)
+    {
+      G_FREE(args->if_modified_since);
+      args->if_modified_since = NULL;
+    }
+
+  if (http_read_from_url_synchronously(args->broker, buffer))
+    {
+      /* ̿ */
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_OK;
+#if DEBUG_NETWORK_MOST
+      fprintf(stderr, "reading from %s succeeded.\n", args->url);
+#endif
+    }
+  else
+    {
+      /* ̿ */
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+#if DEBUG_NETWORK
+      fprintf(stderr, "reading from %s failed.\n", args->url);
+#endif
+    }
+
+  ochusha_async_buffer_active_unref(buffer,
+		"ochusha_network_broker.c: force_read_synchronously()");
+
+ finish_force_read:
+  ochusha_async_buffer_fix(buffer,
+		"ochusha_network_broker.c: force_read_synchronously()");
+  g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
+  g_object_unref(G_OBJECT(buffer));
+}
+
+
+/*
+ * ochusha_network_broker_read_from_cgi
+ *
+ * Ϳ줿URLǡɤ߹ࡣ
+ * offline⡼ɤǤNULL֤
+ *
+ * åˤϿʤޤCGIξHEAD쥹ݥ󥹤Ƥʤ
+ * ȤΤǡƱ⡼ɤȤGETȯɤ߹ࡣ
+ */
+OchushaAsyncBuffer *
+ochusha_network_broker_read_from_cgi(OchushaNetworkBroker *broker,
+				     const char *url)
+{
+  g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker)
+		       && broker->config != NULL && url != NULL, NULL);
+#if DEBUG_NETWORK_MOST
+  fprintf(stderr, "ochusha_network_broker_read_from_cgi: %s\n", url);
+#endif
+
+  if (broker->config->offline)
+    return NULL;
+
+  return ochusha_network_broker_employ_worker_thread(broker, url, NULL,
+					(JobFunc *)force_read_synchronously);
+}
+
+
 /*
  * ochusha_network_broker_get_response_header:
  *
