@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.14 2003/11/08 23:44:33 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.23 2003/11/20 03:14:37 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,6 +43,30 @@
 #include <time.h>
 #include <unistd.h>
 
+#if ENABLE_REGEXP
+# if HAVE_ONIGURUMA
+#  if HAVE_ONIG_ONIGPOSIX_H
+#    include <onig/onigposix.h>
+#  elif HAVE_ONIGPOSIX_H
+#    include <onigposix.h>
+#  else
+#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
+#  endif
+# else
+#  if HAVE_POSIX_REGEX
+#    if HAVE_SYS_TYPES_H
+#      include <sys/types.h>
+#    endif
+#    if HAVE_REGEX_H
+#      include <regex.h>
+#    else
+#      warning "I don't know suitable header file of your posix regex library."
+#    endif
+#  endif
+# endif
+# include "ochusha_utils_jbbs.h"
+#endif
+
 
 #define DEBUG_DAT	0
 #define DEBUG_DAT_MOST	0
@@ -93,12 +117,14 @@ static gboolean ochusha_thread_2ch_preview_response(OchushaBBSThread *thread,
 					EndThreadCallback *end_cb,
 					gpointer callback_data);
 
+static gboolean ochusha_thread_2ch_post_supported(OchushaBBSThread *thread);
+
 static gboolean ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
 					OchushaConfig *config,
 					const OchushaBBSResponse *response);
 
 static char *ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread,
-						     int number);
+						     int from, int to);
 static const char *ochusha_thread_2ch_get_url_to_post(OchushaBBSThread *thread);
 
 
@@ -133,6 +159,9 @@ ochusha_thread_2ch_get_type(void)
 
 static OchushaBBSThreadClass *parent_class = NULL;
 
+#if ENABLE_REGEXP
+static regex_t cgi_query_pattern;
+#endif
 
 static void
 ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
@@ -159,7 +188,13 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
   b_class->remove_cache = ochusha_thread_2ch_remove_cache;
 
   b_class->preview_response = ochusha_thread_2ch_preview_response;
+  b_class->post_supported = ochusha_thread_2ch_post_supported;
   b_class->post_response = ochusha_thread_2ch_post_response;
+
+#if ENABLE_REGEXP
+  if (regcomp(&cgi_query_pattern, "&?([^=]+)=([^&]*)", REG_EXTENDED))
+    fprintf(stderr, "invalid regular expression\n");
+#endif
 }
 
 
@@ -310,6 +345,195 @@ ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread)
 }
 
 
+#if ENABLE_REGEXP
+/*
+ * "../test/read.cgi?bbs=linux&key=987524960&st=109&to=109&nofirst=true"
+ * ߤURLβϡ
+ */
+static gboolean
+ochusha_thread_2ch_check_relative_url(OchushaThread2ch *thread_2ch,
+				      const char *url,
+				      unsigned int *from_p, unsigned int *to_p)
+{
+  unsigned int from = 0;
+  unsigned int to = 0;
+  gboolean is_same_board = FALSE;
+  gboolean is_same_thread = FALSE;
+  OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
+  regmatch_t match[3];
+
+  url = strchr(url, '?');
+  if (url == NULL)
+    return FALSE;
+  url++;
+
+  /* cgi_query_pattern"^([^=]+)=([^&]*)"ɽ */
+  while (regexec(&cgi_query_pattern, url, 2, match, 0) == 0)
+    {
+      int match_len = match[0].rm_eo - match[0].rm_so;
+      const char *name_head = url + match[1].rm_so;
+      int name_len = match[1].rm_eo - match[1].rm_so;
+      const char *value_head = url + match[2].rm_so;
+      int value_len = match[2].rm_eo - match[2].rm_so;
+
+      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
+	{
+	  if (strncmp(thread->board->id, value_head, value_len) == 0)
+	    is_same_board = TRUE;
+	  else
+	    return FALSE;
+	}
+      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
+	{
+	  const char *key = thread->id;
+	  if (strncmp(key, value_head, value_len) == 0)
+	    is_same_thread = TRUE;
+	  else
+	    return FALSE;
+	}
+      else if (g_ascii_strncasecmp("ST", name_head, name_len) == 0
+	       || g_ascii_strncasecmp("START", name_head, name_len) == 0)
+	{
+	  if (value_len > 0)
+	    {
+	      char *value = g_strndup(value_head, value_len);
+	      if (sscanf(value, "%d", &from) != 1)
+		from = 0;
+	      g_free(value);
+	    }
+	}
+      else if (g_ascii_strncasecmp("TO", name_head, name_len) == 0
+	       || g_ascii_strncasecmp("END", name_head, name_len) == 0)
+	{
+	  if (value_len > 0)
+	    {
+	      char *value = g_strndup(value_head, value_len);
+	      if (sscanf(value, "%d", &to) != 1)
+		to = 0;
+	      g_free(value);
+	    }
+	}
+
+      url += match_len;
+    }
+
+  if (!is_same_board || !is_same_thread)
+    return FALSE;
+
+  if (from == 0 && to != 0)
+    from = to;
+  else if (to == 0 && from != 0)
+    to = from;
+  else if (from == 0 && to == 0)
+    {
+      from = 1;
+      to = 1;
+    }
+
+  if (from_p != NULL)
+    *from_p = from;
+
+  if (to_p != NULL)
+    *to_p = to;
+
+  return TRUE;
+}
+
+
+/*
+ * "http://jbbs.shitaraba.com/computer/bbs/read.cgi?BBS=351&KEY=1040452877&START=109&END=109&NOFIRST=TRUE"
+ * ߤURLβϡ
+ */
+static gboolean
+ochusha_thread_2ch_check_absolute_url(OchushaThread2ch *thread_2ch,
+				      const char *url,
+				      unsigned int *from_p, unsigned int *to_p)
+{
+  unsigned int from = 0;
+  unsigned int to = 0;
+  gboolean is_same_board = FALSE;
+  gboolean is_same_thread = FALSE;
+  OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
+  regmatch_t match[3];
+
+  url = strchr(url, '?');
+  if (url == NULL)
+    return FALSE;
+  url++;	/* ?ޤǤФ */
+
+  /* cgi_query_pattern"^([^=]+)=([^&]*)"ɽ */
+  while (regexec(&cgi_query_pattern, url, 2, match, 0) == 0)
+    {
+      int match_len = match[0].rm_eo - match[0].rm_so;
+      const char *name_head = url + match[1].rm_so;
+      int name_len = match[1].rm_eo - match[1].rm_so;
+      const char *value_head = url + match[2].rm_so;
+      int value_len = match[2].rm_eo - match[2].rm_so;
+
+      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
+	{
+	  if (strncmp(thread->board->id, value_head, value_len) == 0)
+	    is_same_board = TRUE;
+	  else
+	    return FALSE;
+	}
+      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
+	{
+	  if (strncmp(thread->id, value_head, value_len) == 0)
+	    is_same_thread = TRUE;
+	  else
+	    return FALSE;
+	}
+      else if (g_ascii_strncasecmp("START", name_head, name_len) == 0
+	       || g_ascii_strncasecmp("ST", name_head, name_len) == 0)
+	{
+	  if (value_len > 0)
+	    {
+	      char *value = g_strndup(value_head, value_len);
+	      if (sscanf(value, "%d", &from) != 1)
+		from = 0;
+	      g_free(value);
+	    }
+	}
+      else if (g_ascii_strncasecmp("END", name_head, name_len) == 0
+	       || g_ascii_strncasecmp("TO", name_head, name_len) == 0)
+	{
+	  if (value_len > 0)
+	    {
+	      char *value = g_strndup(value_head, value_len);
+	      if (sscanf(value, "%d", &to) != 1)
+		to = 0;
+	      g_free(value);
+	    }
+	}
+
+      url += match_len;
+    }
+
+  if (!is_same_board || !is_same_thread)
+    return FALSE;
+
+  if (from == 0 && to != 0)
+    from = to;
+  else if (to == 0 && from != 0)
+    to = from;
+  else if (from == 0 && to == 0)
+    {
+      from = 1;
+      to = 1;
+    }
+
+  if (from_p != NULL)
+    *from_p = from;
+
+  if (to_p != NULL)
+    *to_p = to;
+
+  return TRUE;
+}
+#endif
+
+
 static gboolean
 ochusha_thread_2ch_check_url(OchushaBBSThread *thread, const char *url,
 			     unsigned int *from_p, unsigned int *to_p)
@@ -323,13 +547,28 @@ ochusha_thread_2ch_check_url(OchushaBBSThread *thread, const char *url,
       unsigned int to = 0;
       OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
+      while (strncmp(url + 3, "../", 3) == 0)
+	url += 3;	/* ƬϢ³"../"ɤФ */
+
       base_path = ochusha_thread_2ch_get_base_path(thread_2ch);
       g_return_val_if_fail(base_path != NULL, FALSE);
 
       len = strlen(base_path);
       url += 2;	/* Ƭ".."ɤФ */
+
+#if 0
+      fprintf(stderr, "ochusha_thread_2ch_check_url()\n");
+      fprintf(stderr, "  url: %s\n", url);
+      fprintf(stderr, "  base_path: %s\n", base_path);
+#endif
+
       if (strncmp(base_path, url, len) != 0)
+#if ENABLE_REGEXP
+	return ochusha_thread_2ch_check_relative_url(thread_2ch, url - 2,
+						     from_p, to_p);
+#else
 	return FALSE;
+#endif
 
       sscanf(url + len, "%u-%u", &from, &to);
       if (from < 0)
@@ -347,8 +586,11 @@ ochusha_thread_2ch_check_url(OchushaBBSThread *thread, const char *url,
 
       return TRUE;
     }
-
-  /* XXX: http://̾URLǤå٤ */
+#if ENABLE_REGEXP
+  else
+    return ochusha_thread_2ch_check_absolute_url(OCHUSHA_THREAD_2CH(thread),
+						 url, from_p, to_p);
+#endif
 
   return FALSE;
 }
@@ -396,7 +638,7 @@ ochusha_thread_2ch_get_dat_url(OchushaThread2ch *thread_2ch)
 		       && thread->id != NULL,
 		       NULL);
 
-  if (snprintf(url, PATH_MAX, "%sdat/%s",
+  if (snprintf(url, PATH_MAX, "%sdat/%s.dat",
 	       thread->board->base_url, thread->id) >= PATH_MAX)
     return NULL;
 
@@ -437,7 +679,6 @@ const char *
 ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
 {
   char base_path[PATH_MAX];
-  char *tmp_pos;
   OchushaBBSThread *thread;
 
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch), NULL);
@@ -449,15 +690,41 @@ ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
 
   g_return_val_if_fail(thread->board != NULL && thread->id != NULL, NULL);
 
-  if (snprintf(base_path, PATH_MAX, "/test/read.cgi%s%s",
-	       thread->board->base_path, thread->id) >= PATH_MAX)
-    return NULL;
-
-  tmp_pos = strstr(base_path + 14, ".dat");
-  if (tmp_pos != NULL)
+  switch (thread->board->bbs_type)
     {
-      tmp_pos[0] = '/';
-      tmp_pos[1] = '\0';
+    case OCHUSHA_BBS_TYPE_2CH:
+    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
+      if (snprintf(base_path, PATH_MAX, "/test/read.cgi%s%s/",
+		   ochusha_bulletin_board_get_base_path(thread->board),
+		   thread->id) >= PATH_MAX)
+	return NULL;
+      break;
+
+    case OCHUSHA_BBS_TYPE_MACHIBBS:
+      if (snprintf(base_path, PATH_MAX, "/bbs/read.pl?BBS=%s&KEY=%s",
+		   ochusha_bulletin_board_get_id(thread->board), thread->id)
+	  >= PATH_MAX)
+	return NULL;
+      break;
+
+    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+      if (snprintf(base_path, PATH_MAX, "/bbs/read.cgi%s%s/%s/",
+		   ochusha_bulletin_board_get_base_path(thread->board),
+		   ochusha_bulletin_board_get_id(thread->board),
+		   thread->id) >= PATH_MAX)
+	return NULL;
+      break;
+
+    case OCHUSHA_BBS_TYPE_MITINOKU:
+      if (snprintf(base_path, PATH_MAX, "/read.cgi?bbs=%s&key=%s",
+		   ochusha_bulletin_board_get_id(thread->board), thread->id)
+	  >= PATH_MAX)
+	return NULL;
+      break;
+
+    case OCHUSHA_BBS_TYPE_JBBS:
+    default:	/* ̤ݡ */
+      return NULL;
     }
 
   thread_2ch->base_path = G_STRDUP(base_path);
@@ -476,9 +743,26 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
-  return ochusha_network_broker_read_from_url(broker,
+  switch (thread->board->bbs_type)
+    {
+    case OCHUSHA_BBS_TYPE_2CH:
+    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
+      return ochusha_network_broker_read_from_url(broker,
 				ochusha_thread_2ch_get_dat_url(thread_2ch),
 				thread_2ch->last_modified, mode);
+#if ENABLE_REGEXP
+# if HAVE_ONIGURUMA
+    case OCHUSHA_BBS_TYPE_JBBS:
+    case OCHUSHA_BBS_TYPE_MACHIBBS:
+    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+    case OCHUSHA_BBS_TYPE_MITINOKU:
+      return ochusha_utils_jbbs_fake_responses_source(thread_2ch, broker,
+						      mode);
+# endif
+#endif
+    default:
+      return NULL;
+    }
 }
 
 
@@ -784,6 +1068,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 		buffer->length);
 #endif
 	while (rest_of_data > 0 && rest_of_responses > 0
+	       && i < MAX_RESPONSE	/* XXX: ܤꤹ */
 	       && (buffer_fixed || interval-- > 0)
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
@@ -813,7 +1098,6 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    responses[i] = offset;
 
 	    i++;
-#if 1
 	    if (i > start)
 	      {
 		result = TRUE;
@@ -832,20 +1116,6 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 		if (!result)
 		  break;
 	      }
-#else
-	    if (title_pos != NULL)
-	      {
-		if (i > start)
-		  {
-		    if (each_response_cb != NULL)
-		      result = (*each_response_cb)(thread, i, &response,
-						   callback_data);
-		    rest_of_responses--;
-		  }
-		if (!result)
-		  break;
-	      }
-#endif
 
 	    offset = (eol_pos + 1) - buffer_top;
 	    buffer_top = (char *)buffer->buffer;
@@ -956,23 +1226,68 @@ ochusha_thread_2ch_get_last_modified_utc(OchushaBBSThread *thread)
 
 
 static char *
-ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread, int number)
+ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread, int from,
+					int to)
 {
   OchushaThread2ch *thread_2ch;
   char url[PATH_MAX];
-  const char *base_url;
 
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), NULL);
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
-  base_url = ochusha_thread_2ch_get_base_url(thread_2ch);
-  base_url = strstr(base_url, "/test/read.cgi");
+  switch (thread->board->bbs_type)
+    {
+    case OCHUSHA_BBS_TYPE_2CH:
+      {
+	if (from >= to)
+	  snprintf(url, PATH_MAX, "../test/read.cgi/%s/%s/%d",
+		   ochusha_bulletin_board_get_id(thread->board),
+		   thread->id, from);
+	else
+	  snprintf(url, PATH_MAX, "../test/read.cgi/%s/%s/%d-%d",
+		   ochusha_bulletin_board_get_id(thread->board),
+		   thread->id, from, to);
+	break;
+      }
 
-  if (base_url == NULL)
-    return NULL;
+    case OCHUSHA_BBS_TYPE_MACHIBBS:
+      {
+	if (from >= to)
+	  to = from;
+	snprintf(url, PATH_MAX, "http://%s/bbs/read.pl?BBS=%s&KEY=%s&START=%d&END=%d&NOFIRST=TRUE",
+		 ochusha_bulletin_board_get_server(thread->board),
+		 ochusha_bulletin_board_get_id(thread->board),
+		 thread->id, from, to);
+	break;
+      }
 
-  snprintf(url, PATH_MAX, "..%s%d", base_url, number);
+    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+      {
+	if (from >= to)
+	  to = from;
+	snprintf(url, PATH_MAX, "http://%s%sbbs/read.cgi?BBS=%s&KEY=%s&START=%d&END=%d&NOFIRST=TRUE",
+		 ochusha_bulletin_board_get_server(thread->board),
+		 ochusha_bulletin_board_get_base_path(thread->board),
+		 ochusha_bulletin_board_get_id(thread->board),
+		 thread->id, from, to);
+	break;
+      }
+
+    case OCHUSHA_BBS_TYPE_MITINOKU:
+      {
+	if (from >= to)
+	  to = from;
+	snprintf(url, PATH_MAX, "http://%s/read.cgi?bbs=%s&key=%s&st=%d&to=%d&nofirst=true",
+		 ochusha_bulletin_board_get_server(thread->board),
+		 ochusha_bulletin_board_get_id(thread->board),
+		 thread->id, from, to);
+	break;
+      }
+
+    default:	/* ̤ݡ */
+      return NULL;
+    }
 
   return G_STRDUP(url);
 }
@@ -990,8 +1305,32 @@ ochusha_thread_2ch_get_url_to_post(OchushaBBSThread *thread)
   if (thread_2ch->post_url != NULL)
     return thread_2ch->post_url;
 
-  snprintf(url, PATH_MAX, "%sl10",
-	   ochusha_thread_2ch_get_base_url(thread_2ch));
+  switch (thread->board->bbs_type)
+    {
+    case OCHUSHA_BBS_TYPE_2CH:
+      snprintf(url, PATH_MAX, "%sl10",
+	       ochusha_thread_2ch_get_base_url(thread_2ch));
+      break;
+
+    case OCHUSHA_BBS_TYPE_MACHIBBS:
+    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+      snprintf(url, PATH_MAX, "%s?BBS=%s&KEY=%s&LAST=10",
+	       ochusha_board_2ch_get_read_cgi_url(OCHUSHA_BOARD_2CH(thread->board)),
+	       ochusha_bulletin_board_get_id(thread->board),
+	       thread->id);
+      break;
+
+    case OCHUSHA_BBS_TYPE_MITINOKU:
+      snprintf(url, PATH_MAX, "%s?bbs=%s&key=%s&ls=10",
+	       ochusha_board_2ch_get_read_cgi_url(OCHUSHA_BOARD_2CH(thread->board)),
+	       ochusha_bulletin_board_get_id(thread->board),
+	       thread->id);
+      break;
+
+    default:	/* ̤ݡ */
+      break;
+    }
+
   thread_2ch->post_url = G_STRDUP(url);
   return thread_2ch->post_url;
 }
@@ -1009,6 +1348,16 @@ ochusha_thread_2ch_preview_response(OchushaBBSThread *thread,
 }
 
 
+static gboolean
+ochusha_thread_2ch_post_supported(OchushaBBSThread *thread)
+{
+  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), FALSE);
+  g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(thread->board), FALSE);
+
+  return (thread->board->bbs_type == OCHUSHA_BBS_TYPE_2CH);
+}
+
+
 static gboolean
 ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
 				 OchushaConfig *config,
@@ -1021,7 +1370,7 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
   char *mail = NULL;
   char *message = NULL;
   const char *bbs;
-  char *key = NULL;
+  const char *key;
   char *query = NULL;
   time_t time;
   iconv_t converter;
@@ -1033,6 +1382,9 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
   g_return_val_if_fail(response->mailto != NULL, FALSE);
   g_return_val_if_fail(response->content != NULL, FALSE);
 
+  if (thread->board->bbs_type != OCHUSHA_BBS_TYPE_2CH)
+    return FALSE;	/* ̤ݡ */
+
   converter = iconv_open(ochusha_bulletin_board_get_response_character_encoding(thread->board), "UTF-8");
   g_return_val_if_fail(converter != (iconv_t)-1, FALSE);
 
@@ -1059,13 +1411,7 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
   tmp_string = NULL;
 
   bbs = ochusha_bulletin_board_get_id(thread->board);
-
-  tmp_string = (char *)ochusha_bbs_thread_get_id(thread);
-  g_assert(strstr(tmp_string, ".dat"));
-  key = g_strndup(tmp_string, strstr(tmp_string, ".dat") - tmp_string);
-  tmp_string = NULL;
-  if (key == NULL)
-    goto error_exit;
+  key = thread->id;
 
   if (thread_2ch->date != NULL)
     time = ochusha_utils_get_utc_time(thread_2ch->date);
@@ -1111,8 +1457,6 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
     G_FREE(mail);
   if (message != NULL)
     G_FREE(message);
-  if (key != NULL)
-    G_FREE(key);
   if (query != NULL)
     G_FREE(query);
   iconv_close(converter);
@@ -1132,8 +1476,6 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
     G_FREE(mail);
   if (message != NULL)
     G_FREE(message);
-  if (key != NULL)
-    G_FREE(key);
   if (query != NULL)
     G_FREE(query);
   iconv_close(converter);
