@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.10 2003/11/07 14:23:00 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.11 2003/11/18 21:02:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -90,6 +90,7 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
       char *close_tag;
       OchushaBulletinBoard *board;
       OchushaBulletinBoard *board_by_name;
+      gboolean is_machibbs = FALSE;
 
       if (tag_tail == NULL)
 	return FALSE;	/* Ƥ롩 */
@@ -106,7 +107,13 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
 	/* 2chİʳؤΥ󥫡ˤTARGE°դƤ̣*/
 	char *target = g_strstr_len(cur_pos, close_tag - cur_pos, "TARGET=");
 	if (target != NULL)
-	  goto search_next;
+	  {
+	    /* ޤBBS⥵ݡȤͽ */
+	    if (g_strstr_len(cur_pos, close_tag - cur_pos, "machi.to") != NULL)
+	      is_machibbs = TRUE;
+	    else
+	      goto search_next;
+	  }
       }
 #endif
       name = convert_string(converter, cp932_to_utf8_helper,
@@ -131,6 +138,8 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
 	    }
 
 	  board = ochusha_board_2ch_new(name, url);
+	  if (is_machibbs)
+	    board->bbs_type = OCHUSHA_BBS_TYPE_MACHIBBS;
 
 #if GUESS_2CH_BOARD_URL
 	  base_path = ochusha_bulletin_board_get_base_path(board);
